/*
 * tsbc.c -- An HP-2000 Timeshared BASIC System tape manipulation program.
 *
 * Copyright (c) Scott G. Taylor
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * SCOTT G. TAYLOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Scott G. Taylor shall not
 * be used in advertising or otherwise to promote the sale, use or other dealings
 * in this Software without prior written authorization from Scott G. Taylor.
 */

/*
 * tsbc - Convert a tokenized HP-2000 TSB BASIC file to ASCII.
 */

/*
 *	If you need technical assistance with this program, have any
 *	HP-2000A, HP-2000B, HP-2000C or HP-2000E tapes or images you
 *	can provide for testing, or just want to comment on this program,
 *	please contact Scott G. Taylor at staylor@mrynet.com.
 */

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "config.h"

#include "hptsb.h"
#include "decl.h"
#include "tsb.h"
#include "hptokdecl.h"
#include "hptoken.h"
#include "version.h"

#define MAXENTRIES	32768

uint32_t	nentries = 1;
uint8_t		rentry[1][512];

uint32_t	centries = 0;
uint8_t		centry[MAXENTRIES][512];

uint32_t	last_progID = 0;

/*
 * usage -	Display program usage.
 */

static void
usage (void)
{
 	errorexit (1, "Usage: %s [-vVO] [-d x...] [-s newsuffix] {-I inputlistfile | filename ...}\n",
		progname);
}

int
main (int argc, char **argv)
{
	char		buf[BUFMAX];
	char		listfilename[MAXPATHLEN] = "";
	char		newsuffix[MAXPATHLEN] = ".ascii";
	uint32_t	rlen;
	int		c;
	int		i;
	int		Wfd = -1;

	progname = basename (argv[0]);
	extract = 0;
	only_programs = 1;
	only_csave = 1;
	only_files = 1;
	only_ascii_files = 1;


	/*debug = 0x930;*/
debug=0xffffffff;

	while ((c = getopt(argc, argv, "d:OI:s:vVx")) != -1) {
		switch (c) {
		case 'O':
			overwrite++;
			break;
		case 's':
			strcpy (newsuffix, optarg);
			break;
		case 'v':
			verbose++;
			break;
		case 'x':
			extract++;
			break;
		case 'd':
		    {
			struct debuglist	*dl;

			for (i = 0; i < strlen(optarg); i++) {
				dl = debuglist;
				while (dl->level != 0) {
					if (optarg[i] == dl->letter) {
						debug += (2 << dl->level);
						break;
					}
					dl++;
				}
				if (dl->level == 0) {
					eprintf ("Invalid debug level: %c\n", optarg[i]);
					exit(99);
				}
			}
			break;
		    }
		case 'I':
			strcpy(listfilename, optarg);
			break;
		case 'V':
			eprintf ("%s\n", version);
			break;
		default:
			errorexit (1, "Invalid option: -%c\n", c);
		}
	}

	argc -= optind;
	argv += optind;

	if (listfilename[0] != 0) {
		uint8_t	*tbp;
		FILE	* flfd;
		flfd = fopen (listfilename, "r");
		if (flfd == NULL)
			errorexit (errno, "Error opening list file %s: %s\n",
					listfilename, strerror(errno));
		while ((tbp = (uint8_t *) fgets((char *) centry[centries], 511, flfd)) != NULL) {
			dprintf (DEBUG_TABLES, "Adding list entry #%d: %s\n", centries, centry[centries]);
			for (i = 0; i < strlen((char *) centry[centries]); i++) {
				if (centry[centries][i] == '\n')
					centry[centries][i] = 0;
				else
					centry[centries][i] = centry[centries][i];
			}
			centries++;
		}
	} else if (argc == 0) {
		centries = 1;
		strcpy((char *) centry[0], "*");
	} else while (argc) {
		dprintf (DEBUG_SYSTEM, "Saving requested entry %s in slot %d\n", argv[0], centries);
		strcpy ((char *) centry[centries], argv[0]);
		for (i = 0; i < strlen((char *) centry[centries]); i++)
			centry[centries][i] = centry[centries][i];
		centries++;
		argc -= 1;
		argv++;
	}
	nentries = 1;
	strcpy((char *) rentry[0], "*");

	if (argc != 0)
		usage();

	fnno = 0;
	convert = 1;
	Pentry.type = tape_mach_type = MACH_TYPE_SET_ACCESSB;

	if (verbose > 1)
		eprintf (" ID      NAME        DATE            LENGTH    DEV/ADDR   RECORD\n");

	while (fnno < centries) {
		Wfd = open ((char *) centry[fnno], O_RDONLY);
		if (Wfd < 0)
			errorexit (errno, "Error opening input file %s: %s\n",
					centry[fnno], strerror(errno));

		dprintf (DEBUG_SYSTEM, "MAIN: Reading file %d: %s\n", fnno + 1, centry[fnno]);
		
		if ((rlen = read (Wfd, buf, MAXBSIZE)) < 24)
			errorexit (-21, "Can't read header bytes from file %s: %s\n",
					centry[fnno], strerror(errno));

		memcpy (Pentry.header, buf, 24);

		if (extract)
			sprintf (Nfilename, "%s%s", centry[fnno], newsuffix);
		load_program_entry (0, (uint8_t *) buf, rlen, MACH_TYPE_SET_ACCESSB);
		load_program_entry (2, (uint8_t *) buf, 0, MACH_TYPE_SET_ACCESSB);
		fnno++;

	}
	close (Wfd);
	n_list(DEBUG_MEMORY);
	exit (0);
}
